package com.bea.wlevs.demo.airline.client.model;

import java.io.Serializable;
import java.util.Calendar;
import java.util.LinkedList;

public class Container implements Serializable, ModelState {
static final long serialVersionUID = 1;
	
	private static String[] stateText = {"Unknown", "Ready for Bags", "On Board" };
	
	/** This container is accepting bags for the flight */
	public static final int CONTAINERSTATE_READYFORBAGS = 1;
	
	/** The container is closed and is ready to be placed onto the plane */
	public static final int CONTAINERSTATE_ONBOARD = 2;
	
	/** 
	 * This is the highest value possible for the BAGSTATE. Used to 
	 * ensure that the bag state value is alway within logical limits.
	 */
	private static final int CONTAINERSTATE_MAX = CONTAINERSTATE_ONBOARD;
	
	/** This must be one the the CONTAINERSTATE_* values defined in this class */
	private int eventState;
	
	/** Has this event been read and posted to the event server? */
	private boolean hasUnpostedChanges = true;
	
	/** The time that the current state for this event was set. */
	private Calendar eventTime = null;
	
	/** Use to generate ID number for this event class */
	private static int idGenerator = 5000;
	
	/** The ID of this specific event */
	private int id;
	
	/** The flight to which this container is destined */
	private Flight flight = null;
	
	/** The bags that are loaded into this container */
	LinkedList<Bag> bagList = new LinkedList<Bag>();
	
	
	public Container(Flight flight) {
		this.id = Container.idGenerator++;
		this.flight = flight;
		this.eventState = CONTAINERSTATE_READYFORBAGS;
		this.hasUnpostedChanges = true;
		this.eventTime = Calendar.getInstance();
	}
	
	/**
	 * Load the specified bag onto this container
	 * @param bag
	 */
	public void loadBag(Bag bag) {
		bagList.add(bag);
	}
	
	/**
	 * Show that this event has been read
	 */
	public void changesArePosted() {
		this.hasUnpostedChanges = false;
	}
	
	/**
	 * See if this event has been read by the AirportStateCoordinator
	 */
	public boolean hasUnpostedChanges() {
		return this.hasUnpostedChanges;
	}
	
	/**
	 * Promote this event to the next state in its lifecycle.
	 */
	public void nextState() {
		if(hasUnpostedChanges) {
			throw new IllegalStateException("Trying to change state when there are unposted changes.  Current state is:"+this.getEventState());
		}
		// Update the event time stamp
		this.eventTime = Calendar.getInstance();
		eventState++;
		if(eventState > CONTAINERSTATE_MAX)
			eventState = CONTAINERSTATE_MAX;
	}
	
	/**
	 * Get the timestamp of this event
	 */
	public Calendar getEventTime() {
		return eventTime;
	}
	
	/**
	 * Get the event type.
	 * @return <pre>PEOPLEEVENT_CHECKIN</pre>, <pre>PEOPLEEVENT_BOARDPLANE</pre>
	 */
	public int getEventState() {
		return eventState;
	}
	
	/**
	 * Get the flight for which this bag is destined.
	 * @return The flight that this bag should be on.
	 */
	public Flight getFlight() {
		return this.flight;
	}
	
	
	/**
	 * Set the event type.
	 * @param eventType This must be one of the following values: 
	 * <pre>PEOPLEEVENT_CHECKIN</pre>, <pre>PEOPLEEVENT_BOARDPLANE</pre>
	 */
	public void setEventState(int eventState) {
		this.eventState = eventState;
		if(this.eventState > Container.CONTAINERSTATE_MAX)
			this.eventState = Container.CONTAINERSTATE_MAX;
	}
	
	
	/**
	 * Get the unique ID that represents this person
	 * @return The ID of the person
	 */
	public int getID() {
		return id;
	}
	
	
	/**
	 * Get the type (read: name) of the event.
	 * @return A String that holds the event type.
	 */
	public String getEventType() {
		return "Container";
	}
	
	/**
	 * Get the string version of this event
	 */
	public String toString() {
		return "Container [" + this.getID() + "] for flight " + flight + " " + 
		stateText[this.getEventState()];
	}

	/**
	 * Sets the flight for which this bag is destined.
	 * @param flight The flight that should carry this bag when it leaves.
	 */
	public void setFlight(Flight flight) {
		this.flight = flight;
	}
}
