package com.bea.wlevs.demo.airline.client.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.LinkedList;
import java.util.List;

public class Flight implements Serializable, ModelState {
	static final long serialVersionUID = 1;
	
	/** The flight has been created */
	public static int FLIGHTEVENT_CREATED = 1;
	
	/** Passengers may board the plane now */
	public static int FLIGHTEVENT_BOARDING = 2;
	
	/** Crew checks for stragglers stuck in security */
	public static int FLIGHTEVENT_BOARDINGFINAL = 3;	
	
	/** The door is closed. No one may board and the plane is headed to the runway */
	public static int FLIGHTEVENT_BOARDINGCLOSED = 4;
	
	/** The door is closed. No one may board and the plane is headed to the runway */
	public static int FLIGHTEVENT_DEPARTED = 5;

	/** The aircraft is in flight */
	public static int FLIGHTEVENT_WHEELSUP = 6;
	
	/** This tracks the current event state */
	private int eventState = FLIGHTEVENT_CREATED;
	
	private static String[] statusText = {"Unknown", "Pre-boarding", "Boarding", "BoardingFinal", 
		"Doors Closed", "Departed", "Takeoff" };
	private static int idGenerator = 100;
	private int id;
	private boolean onTime = true;
	private Calendar departureTime = null;
	private Calendar eventTime = null;
	private LinkedList<People> passengerList = new LinkedList<People>();
	private Container container = null;
	
	private boolean hasUnpostedChanges;

	/** Used to tag flight as one to track in detail.  Set by demo UI */  
	private boolean trackFlightDetails = false;
	
	// for simulation
	boolean delayTakeoff;
	
	// Each plane carries 100 passengers at most
	public static final int MAX_SEATING = 100;
	
	/**
	 * Creates a Flight and automatically assigns a flight number
	 *
	 */
	public Flight() {
		id = Flight.idGenerator++;
		// All flights depart 10 minutes after they are created
		departureTime = Calendar.getInstance();
		departureTime.add(Calendar.MINUTE, 10);
		onTime = true;
		container = new Container(this);
		eventTime = Calendar.getInstance();
		this.hasUnpostedChanges = true;
	}
	
	/**
	 * Get the departure time of this flight
	 * @return The scheduled departure time for the flight
	 */
	public Calendar getDepartureTime() {
		return departureTime;
	}


	/**
	 * Adds a delay, in minutes, to the departure time of a flight.
	 * @param departureTime
	 */
	public void delayFlight(int delayInMinutes) {
		this.onTime = false;	// The flight is no longer on time
		this.departureTime.add(Calendar.MINUTE, delayInMinutes);
	}


	/**
	 * Used for test purposes only
	 * @param args
	 */
	public static void main(String[] args) {
		for(int i = 0; i < 5; i++) {
			Flight evt = new Flight();
			System.out.println("Generated flight " + evt.getID());
		}
	}


	/**
	 * Get the flight ID number for this flight.
	 * @return The flight ID number
	 */
	public int getID() {
		return id;
	}
	
	/**
	 * Get the type (read: name) of the event.
	 * @return A String that holds the event type.
	 */
	public String getEventType() {
		return "Flight";
	}
	
	/**
	 * Checks to see if the flight is on time or not.
	 * @return
	 */
	public boolean isOnTime() {
		return onTime;
	}

	/**
	 * Check to see if this flight has any remaining seats open.
	 * @return <pre>true</pre> if there is an open seat for this flight. 
	 * Otherwise it returns <pre>false</pre>
	 */
	public boolean hasOpenSeats() {
		return(passengerList.size() < MAX_SEATING);
	}
	
	/**
	 * A passenger has boarded this flight.
	 * @param evt A People that represents the person wishing to fly on
	 * the aircraft.
	 */
	public void boardPassenger(People evt) {
		passengerList.add(evt);
	}
	
	
	/**
	 * Promote this event to the next state in its lifecycle.
	 */
	public void nextState() {
		if(hasUnpostedChanges) {
			throw new IllegalStateException("Trying to change state when there are unposted changes.  Current state is:"+this.getEventState());
		}
		// Update the event time stamp
		this.eventTime = Calendar.getInstance();
		eventState++;
	}
	
	public List<People> getPassengers() {
		//System.out.println("Flight:passenger list size:"+passengerList.size());
		return new ArrayList<People>(passengerList);
	}
	
	/**
	 * Get the current state of this event.
	 * @return The current event state. This will be one of the following values:
	 * <pre>FLIGHTEVENT_CREATED</pre>, <pre>FLIGHTEVENT_BOARDING</pre>, <pre>FLIGHTEVENT_BOARDINGFINAL</pre>,
	 * <pre>FLIGHTEVENT_BOARDINGCLOSED</pre>, <pre>FLIGHTEVENT_WHEELSUP</pre>
	 */
	public int getEventState() {
		return eventState;
	}

	/**
	 * Get the timestamp of this event
	 */
	public Calendar getEventTime() {
		return eventTime;
	}
	
	public String getStatusText() {
		return statusText[eventState];
	}
	
	public String toString() {
		String result = "Flight " + this.id + " : " + statusText[this.eventState];
		return result;
	}

	/**
	 * Get the COntainerEvent that is associated with this flight.
	 * @return The container that contains the bags for this flight.
	 */
	public Container getContainer() {
		return container;
	}

	public void changesArePosted() {
		this.hasUnpostedChanges = false;
		
	}

	public boolean hasUnpostedChanges() {
		return this.hasUnpostedChanges;
	}

	public boolean getDelayTakeoff() {
		return delayTakeoff;
	}

	public void setDelayTakeoff(boolean delayTakeoff) {
		this.delayTakeoff = delayTakeoff;
	}

	public boolean isTrackFlightDetails() {
		return trackFlightDetails;
	}

	public void setTrackFlightDetails(boolean trackFlightDetails) {
		this.trackFlightDetails = trackFlightDetails;
	}

}
