package com.bea.wlevs.demo.airline.client.model;

import java.io.Serializable;
import java.util.Calendar;
import java.util.Iterator;
import java.util.LinkedList;

public class People implements Serializable, ModelState {
	static final long serialVersionUID = 1;
	
	private String[] stateText = {"Unknown", "Checked In", "In Security", "Screened", "Onboard" };
	
	private LinkedList<Bag> baggage = new LinkedList<Bag>();
	
	/** The passenger has checked in and is ticketed */
	public static final int PEOPLESTATE_CHECKIN = 1;
	
	/** The passenger is in security */
	public static final int PEOPLESTATE_ENTER_SECURITY = 2;

	/** The passenger has cleared security */
	public static final int PEOPLESTATE_EXIT_SECURITY = 3;

	/** The passenger has boarded the plane */
	public static final int PEOPLESTATE_BOARDPLANE = 4;
	
	/** This is the state with the highest value. Used for sanity checking later on */
	private static final int PEOPLESTATE_MAX = PEOPLESTATE_BOARDPLANE;
	
	/** The ifrst name of the passenger */
	private String firstName;
	
	/** The last name of the passenger */
	private String lastName;
	
	/** The time that the current state for this event was set. */
	private Calendar eventTime = null;
	
	/** The flight that this person is going to board */
	private Flight flight = null;
	
	private Boolean delayInSecurity = false;
	
	/**
	 * This must be one the the PEOPLESTATE_* values defined in this class
	 */
	private int eventState;
	
	private static int idGenerator = 1000;
	
	private int id;

	private boolean hasUnpostedChanges;
	
	/**
	 * Creates a People onbject
	 * @param firstName The first name of the passenger
	 * @param lastName The last name of the passenger
	 */
	public People(String firstName, String lastName) {
		this.id = People.idGenerator++;
		this.firstName = firstName;
		this.lastName = lastName;
		this.eventState = PEOPLESTATE_CHECKIN;
		this.eventTime = Calendar.getInstance();
		hasUnpostedChanges = true;
		// People often come with baggage. Generate 0 - 2 pieces of baggage
		// for checking in.
		double d = Math.random();
		int numBags = (int)Math.round(d * 2.0 ); 
		//System.out.println("Passenger " + firstName + " " + lastName + " checking in with " + numBags + " bags");
		for(int x = 0; x < numBags; x++) {
			baggage.add(new Bag(this));
		}
	}
	
	public String getStatusText() {
		return stateText[eventState];
	}
	
	/**
	 * Gets the baggage events for this person
	 * @return The baggage for this person
	 */
	public LinkedList<Bag> getBaggage() {
		return baggage;
	}
	
	/**
	 * Get the timestamp of this event
	 */
	public Calendar getEventTime() {
		return eventTime;
	}
	
	/**
	 * Get the event type.
	 * @return <pre>PEOPLEEVENT_CHECKIN</pre>, <pre>PEOPLEEVENT_BOARDPLANE</pre>
	 */
	public int getEventState() {
		return eventState;
	}
	
	/**
	 * Get the type (read: name) of the event.
	 * @return A String that holds the event type.
	 */
	public String getEventType() {
		return "People";
	}
	
	
	/**
	 * Gets the first name for this passeneger
	 * @return The first name of the person
	 */
	public String getFirstName() {
		return firstName;
	}
	
	/**
	 * Get the unique ID that represents this person
	 * @return The ID of the person
	 */
	public int getID() {
		return id;
	}
	
	
	
	/**
	 * Get the last name for this person
	 * @return The last name
	 */
	public String getLastName() {
		return lastName;
	}
	
	/**
	 * Get the flight that this poerson is going to take.
	 * @return The flight they are going to take.
	 */
	public Flight getFlight() {
		return flight;
	}
	
	/**
	 * Promote this event to the next state in its lifecycle.
	 */
	public void nextState() {
		if(hasUnpostedChanges) {
			throw new IllegalStateException("Trying to change state when there are unposted changes.  Current state is:"+this.getEventState());
		}		// Update the event time stamp
		this.eventTime = Calendar.getInstance();
		eventState++;
	}
	
	/**
	 * Set the event type.
	 * @param eventState This must be one of the following values: 
	 * <pre>PEOPLESTATE_CHECKIN</pre>, <pre>PEOPLESTATE_BOARDPLANE</pre>
	 */
	public void setEventState(int eventState) {
		this.eventState = eventState;
		if(this.eventState > People.PEOPLESTATE_MAX)
			this.eventState = People.PEOPLESTATE_MAX;
	}
	
	
	/**
	 * Sets the passengers first name
	 * @param firstName The new first name
	 */
	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}
	
	
	/**
	 * Set the flight that this person is going to take
	 * @param flight
	 */
	public void setFlight(Flight flight) {
		this.flight = flight;
		
		// Set the flight for this passengers bags also.
		Iterator iter = baggage.iterator();
		while(iter.hasNext()) {
			Bag bag = (Bag)iter.next();
			bag.setFlight(flight);
		}
	}
	
	
	/**
	 * Sets the last name for the passenger
	 * @param lastName The new last name for the passenger
	 */
	public void setLastName(String lastName) {
		this.lastName = lastName;
	}


	/**
	 * Get the string version of this event
	 */
	public String toString() {
		return "[" + this.getID() + "] " + this.getFirstName() + " " + 
		this.getLastName() + " " + stateText[this.getEventState()];
	}

	public void changesArePosted() {
		this.hasUnpostedChanges = false;
		
	}

	public boolean hasUnpostedChanges() {
		return this.hasUnpostedChanges;
	}

	public Boolean isDelayInSecurity() {
		return delayInSecurity;
	}

	public void setDelayInSecurity(Boolean delayInSecurity) {
		this.delayInSecurity = delayInSecurity;
	}
	
}
