package com.bea.wlevs.demo.airline.client.simulation;

import java.util.Random;
import java.util.concurrent.DelayQueue;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import com.bea.wlevs.demo.airline.client.model.Bag;
import com.bea.wlevs.demo.airline.client.model.Container;
import com.bea.wlevs.demo.airline.client.model.Flight;
import com.bea.wlevs.demo.airline.client.model.People;
import com.bea.wlevs.demo.airline.client.util.NameGenerator;

import java.util.concurrent.TimeUnit;


public class Airport {

	private final int FLIGHT_BOARDING_START = 10;

	private final int FLIGHT_CHECKIN_STOP = 15;

	private final int FLIGHT_BOARDING_FINAL = 20;
	
	private final int FLIGHT_BOARDING_STOP = 25; /** not used */
	
	private final int FLIGHT_CLOSE = 25;

	private final int LOAD_CONTAINER = 30;

	private final int FLIGHT_DEPART = 35;

	private final int FLIGHT_TAKEOFF = 40;

	private final int BAG_SCAN_DELAY = 5;

	private final int BAG_IN_CONTAINER_DELAY = 10;

	private final int FLIGHT_TAKEOFF_DELAY = 10;

	private DelayQueue<ScheduledEvent> scheduledEvents_;

	private ExecutorService eventConsumers_;

	private ExecutorService simulation_;

	private NameGenerator nameGenerator_ = NameGenerator.getInstance();

	private Random random_ = new Random();
	
	public static void main(String[] args) {
		Airport simulation = new Airport();
		simulation.start();
	}

	public Airport() {
	}

	public Flight createFlight() {
		Flight flight = new Flight();
		while (flight.hasOpenSeats()) {
			People passenger = new People(nameGenerator_.getFirstName(),
					nameGenerator_.getLastName());
			passenger.setFlight(flight);
			flight.boardPassenger(passenger);
		}
		return flight;
	}

	/**
	 * Schedule the events for the flight
	 * 
	 * @param flight
	 */
	public void scheduleFlight(Flight flight) {
		// open
		scheduledEvents_.offer(new ScheduledEvent(flight, 0, true));
		// boarding
		scheduledEvents_.offer(new ScheduledEvent(flight, FLIGHT_BOARDING_START));
		// final
		scheduledEvents_.offer(new ScheduledEvent(flight, FLIGHT_BOARDING_FINAL));
		// closed
		scheduledEvents_.offer(new ScheduledEvent(flight, FLIGHT_CLOSE));
		// load container
		Container container = flight.getContainer();
		scheduledEvents_.offer(new ScheduledEvent(container, 0, true));
		scheduledEvents_.offer(new ScheduledEvent(container, LOAD_CONTAINER));
		// depart
		scheduledEvents_.offer(new ScheduledEvent(flight, FLIGHT_DEPART));
		// takeoff
		if (flight.getDelayTakeoff()) {
			scheduledEvents_.offer(new ScheduledEvent(flight, FLIGHT_TAKEOFF + FLIGHT_TAKEOFF_DELAY));
		} else {
			scheduledEvents_.offer(new ScheduledEvent(flight, FLIGHT_TAKEOFF));
		}
		for (People passenger : flight.getPassengers()) {
			int checkInDelay = random_.nextInt(FLIGHT_CHECKIN_STOP - 1) + 1;

			scheduledEvents_.offer(new ScheduledEvent(passenger, checkInDelay,
					true));
			// Add event into and out of security.  We have 10 seconds to fill between
			// check-in and boarding.  We split the interval to pick seconds 2-4 to enter
			// and 6-8 to exit.
			int enterSecurity = random_.nextInt(3) + 2; // 0,1,2 + 2 = 2,3,4
			scheduledEvents_.offer(new ScheduledEvent(passenger, checkInDelay + enterSecurity));
			int exitSecurity = random_.nextInt(3) + 6; // 0,1,2 + 6 = 6,7,8
			scheduledEvents_.offer(new ScheduledEvent(passenger, checkInDelay + exitSecurity));
			
			// next event will be to board
			int boardingDelay = checkInDelay + FLIGHT_BOARDING_FINAL - FLIGHT_CHECKIN_STOP;
			/*
			System.out.println(
					"[passenger:"+passenger.getID()+
					"][checkInDelay:"+checkInDelay +
					"][enterSecurityDelay:"+enterSecurity +
					"][exitSecurityDelay:"+exitSecurity +
					"][boardingDelay:"+boardingDelay +
					"]"
			);
			*/
			
			scheduledEvents_.offer(new ScheduledEvent(passenger, boardingDelay));
			for (Bag bag : passenger.getBaggage()) {
				int scanDelay = random_.nextInt(BAG_SCAN_DELAY - 1) + 1;
				// initial bag check
				scheduledEvents_.offer(new ScheduledEvent(bag, checkInDelay,true));
				// bag scan
				scheduledEvents_.offer(new ScheduledEvent(bag, checkInDelay + scanDelay));
				// bag in container
				scheduledEvents_.offer(new ScheduledEvent(bag, checkInDelay + scanDelay + BAG_IN_CONTAINER_DELAY));
			}
		}
	}

	public void start() {
		start(0);
	}

	public void start(int concurrentFlights) {
		System.out.println("Starting with concurrentFlights:" + concurrentFlights);

		scheduledEvents_ = new DelayQueue<ScheduledEvent>();
		
		eventConsumers_ = Executors.newSingleThreadExecutor();
		eventConsumers_.execute(new FlightEventQueueReader(scheduledEvents_));

		simulation_ = Executors.newSingleThreadExecutor();
		simulation_.execute(new SimulationRunner(this, concurrentFlights));

	}

	public void shutdown() {
		System.out.println("Shutting down threads from Thread:"
				+ Thread.currentThread().getName() + " SecurityManager:"
				+ System.getSecurityManager());
		if (eventConsumers_ != null) {
			eventConsumers_.shutdownNow();
		}
		if (simulation_ != null) {
			simulation_.shutdownNow();
		}
		System.out.println("Done");
	}

	public static class SimulationRunner implements Runnable {
		private Airport simulation_;

		private int concurrentFlights_;

		public SimulationRunner(Airport simulation, int concurrentFlights) {
			simulation_ = simulation;
			concurrentFlights_ = concurrentFlights;
		}

		public void run() {
			if(concurrentFlights_ == 0) {
				System.out.println("No concurrent flights.  Returning from run()");
				return;
			}
			long sleepTime = (simulation_.FLIGHT_TAKEOFF / concurrentFlights_) * 1000;
			System.out.println("SimulationRunner run() [concurrentFlights:"
					+ concurrentFlights_ + "][sleepTime:" + sleepTime + "]");
			while (true) {
				try {
					Flight flight = simulation_.createFlight();
					/*
					System.out.println("SimulationRunner scheduling flight:"
							+ flight.getID() + " using Thread:"
							+ Thread.currentThread().getName());
					*/
					simulation_.scheduleFlight(flight);
					TimeUnit.MILLISECONDS.sleep(sleepTime);
				} catch (InterruptedException e) {
					System.out
							.println("SimulationRunner.run() interrupted concurrent flight loop");
					return;
				}
			}
		}
	}
}
