package com.bea.wlevs.demo.airline.adapter;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

import javax.jms.JMSException;
import javax.jms.MapMessage;
import javax.jms.Message;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.InitializingBean;
import com.bea.wlevs.adapters.jms.api.InboundMessageConverter;
import com.bea.wlevs.ede.api.EventBuilder;
import com.bea.wlevs.ede.api.EventType;
import com.bea.wlevs.ede.api.EventTypeRepository;
import com.bea.wlevs.util.Service;

/**
 * 
 * The default JMS Adapter assumes that all events from a destination  
 * (or destination/selector) get mapped to the same event type.  This implementation
 * uses a configurable message property to define the event type on a per message
 * basic.  It is similar to the approach of using a selector, but avoids the need
 * to deploy an adapter instance per event type.
 * 
 * TODO: might use an init-method (instead of implementing InitializaingBean) 
 * in the Spring config, but if it's not there, then initialization might not happen.
 */
public class InboundJmsDynamicMapMessageConverter implements InboundMessageConverter, InitializingBean {

	/** Key in JMS MapMessage where event type is stored */ 
	private String eventTypeJmsPropertyName_;
	
	/** Need this to create events */ 
	private EventTypeRepository eventTypeRepository_;

	Log log_ = LogFactory.getLog(InboundJmsDynamicMapMessageConverter.class);
	
	/**
	 * Initialized by runtime
	 * @param repository
	 */
	@Service(filter = EventTypeRepository.SERVICE_FILTER)
	public void setEventTypeRepository(EventTypeRepository repository) {
		this.eventTypeRepository_ = repository;
	}
	
	/**
	 * Initialized by Spring
	 * This is the property name where the inbound Event Type Name is stored.
	 * @param propertyName
	 */
	public void setEventTypeJmsPropertyName(String propertyName) {
		eventTypeJmsPropertyName_ = propertyName;
	}
		
	/**
	 * Implement InboundMessageConverter.convert(Message msg)
	 */
	public List<Object> convert(Message jmsMessage) {
		List events = null;
		if(!(jmsMessage instanceof MapMessage)) {
			throw new IllegalArgumentException("Received message that is not a JMSMap message.  Type was:"+jmsMessage.getClass().getName());
		}
		return convertMessageToEvents((MapMessage)jmsMessage);
	}

	private List<Object> convertMessageToEvents(MapMessage mapMessage) {
		String eventTypeName = null;		
		EventType eventType = null;
		EventBuilder eventBuilder = null;
		try {
			eventTypeName = mapMessage.getStringProperty(eventTypeJmsPropertyName_);				
			eventType = eventTypeRepository_.getEventType(eventTypeName);
			// We need an instance for each invocation.  
			eventBuilder = eventType.getEventBuilderFactory().createBuilder();
			assert(eventTypeName != null);
			assert(eventType != null);
			
			Enumeration<String> mapPropertyNames = mapMessage.getMapNames();
			while(mapPropertyNames.hasMoreElements()) {
				String propertyName = mapPropertyNames.nextElement();
				Object propertyValue = mapMessage.getObject(propertyName);
				eventBuilder.put(propertyName, propertyValue);
			}
			//TODO: do we care what type this returns?
			Object event = eventBuilder.createEvent();
			ArrayList<Object> eventList =  new ArrayList<Object>(1);
			eventList.add(event);
			return eventList;
		} catch (JMSException jmse) {
			throw new RuntimeException("JMS exception creating events:"+jmse);
		}
	}
	
	/**
	 * Check that configuration is complete
	 */
	public void afterPropertiesSet() throws Exception {
		if(eventTypeJmsPropertyName_ == null) {
			throw new IllegalStateException("eventTypeJmsPropertyName property must be set");
		}
		log_.info(
				"afterPropertiesSet():"+
						"[eventTypeJmsPropertyName:"+eventTypeJmsPropertyName_ + "]");
	}
}